@@ -42,7 +42,7 @@ class Work:
             },
         depends=['invoiced_duration', 'type'])
     invoice_method = fields.Function(fields.Selection(INVOICE_METHODS,
-            'Invoice Method'), 'get_invoice_method')
+            'Invoice Method'), 'on_change_with_invoice_method')
     invoiced_duration = fields.Function(fields.TimeDelta('Invoiced Duration',
             'company_work_time',
             states={
@@ -96,7 +96,9 @@ class Work:
         default.setdefault('invoice_line', None)
         return super(Work, cls).copy(records, default=default)
 
-    def get_invoice_method(self, name):
+    @fields.depends('type', 'project_invoice_method',
+        'parent', '_parent_parent.invoice_method')
+    def on_change_with_invoice_method(self, name=None):
         if self.type == 'project':
             return self.project_invoice_method
         elif self.parent:
@@ -141,7 +143,7 @@ class Work:
     @staticmethod
     def _get_duration_to_invoice_effort(works):
         return dict((w.id, w.effort_duration) for w in works
-            if w.state == 'done' and not w.invoice_line)
+            if w.state == 'done' and not w.invoice_line and w.effort_duration)
 
     @staticmethod
     def _get_duration_to_invoice_progress(works):
@@ -418,6 +420,7 @@ class Work:
     def _group_lines_to_invoice_key(self, line):
         "The key to group lines"
         return (('product', line['product']),
+            ('unit', line['unit']),
             ('unit_price', line['unit_price']),
             ('description', line['description']))
 
@@ -425,23 +428,27 @@ class Work:
         "Return a invoice line for the lines"
         pool = Pool()
         InvoiceLine = pool.get('account.invoice.line')
-        ModelData = pool.get('ir.model.data')
         Uom = pool.get('product.uom')
 
-        hour = Uom(ModelData.get_id('product', 'uom_hour'))
         quantity = sum(l['quantity'] for l in lines)
         product = key['product']
 
         invoice_line = InvoiceLine()
         invoice_line.type = 'line'
-        invoice_line.quantity = Uom.compute_qty(hour, quantity,
-            product.default_uom)
-        invoice_line.unit = product.default_uom
-        invoice_line.product = product
         invoice_line.description = key['description']
         invoice_line.account = product.account_revenue_used
-        invoice_line.unit_price = Uom.compute_price(hour, key['unit_price'],
-            product.default_uom)
+        if (key['unit']
+                and key['unit'].category == product.default_uom.category):
+            invoice_line.product = product
+            invoice_line.unit_price = Uom.compute_price(
+                key['unit'], key['unit_price'], product.default_uom)
+            invoice_line.quantity = Uom.compute_qty(
+                key['unit'], quantity, product.default_uom)
+            invoice_line.unit = product.default_uom
+        else:
+            invoice_line.unit_price = key['unit_price']
+            invoice_line.quantity = quantity
+            invoice_line.unit = key['unit']
 
         taxes = []
         pattern = invoice_line._get_tax_rule_pattern()
@@ -464,6 +471,12 @@ class Work:
         return []
 
     def _get_lines_to_invoice_effort(self):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        Uom = pool.get('product.uom')
+
+        hour = Uom(ModelData.get_id('product', 'uom_hour'))
+
         if (not self.invoice_line
                 and self.effort_hours
                 and self.state == 'done'):
@@ -474,6 +487,7 @@ class Work:
             return [{
                     'product': self.product,
                     'quantity': self.effort_hours,
+                    'unit': hour,
                     'unit_price': self.list_price,
                     'origin': self,
                     'description': self.name,
@@ -506,6 +520,7 @@ class Work:
             return [{
                     'product': self.product,
                     'quantity': quantity,
+                    'unit': hour,
                     'unit_price': self.list_price,
                     'origin': invoiced_progress,
                     'description': self.name,
@@ -514,6 +529,11 @@ class Work:
         return []
 
     def _get_lines_to_invoice_timesheet(self):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        Uom = pool.get('product.uom')
+
+        hour = Uom(ModelData.get_id('product', 'uom_hour'))
         if (self.timesheet_works
                 and any(tw.timesheet_lines for tw in self.timesheet_works)):
             if not self.product:
@@ -523,6 +543,7 @@ class Work:
             return [{
                     'product': self.product,
                     'quantity': l.hours,
+                    'unit': hour,
                     'unit_price': self.list_price,
                     'origin': l,
                     'description': self.name,
